@@ -60,7 +60,7 @@ class Slot(object):
 class Scraper(object):
 
     def __init__(self, crawler):
-        self.slots = {}
+        self.slot = None
         self.spidermw = SpiderMiddlewareManager.from_crawler(crawler)
         itemproc_cls = load_object(crawler.settings['ITEM_PROCESSOR'])
         self.itemproc = itemproc_cls.from_crawler(crawler)
@@ -72,14 +72,12 @@ class Scraper(object):
     @defer.inlineCallbacks
     def open_spider(self, spider):
         """Open the given spider for scraping and allocate resources for it"""
-        assert spider not in self.slots, "Spider already opened: %s" % spider
-        self.slots[spider] = Slot()
+        self.slot = Slot()
         yield self.itemproc.open_spider(spider)
 
     def close_spider(self, spider):
         """Close a spider being scraped and release its resources"""
-        assert spider in self.slots, "Spider not opened: %s" % spider
-        slot = self.slots[spider]
+        slot = self.slot
         slot.closing = defer.Deferred()
         slot.closing.addCallback(self.itemproc.close_spider)
         self._check_if_closing(spider, slot)
@@ -87,15 +85,14 @@ class Scraper(object):
 
     def is_idle(self):
         """Return True if there isn't any more spiders to process"""
-        return not self.slots
+        return not self.slot
 
     def _check_if_closing(self, spider, slot):
         if slot.closing and slot.is_idle():
-            del self.slots[spider]
             slot.closing.callback(spider)
 
     def enqueue_scrape(self, response, request, spider):
-        slot = self.slots[spider]
+        slot = self.slot
         dfd = slot.add_response_request(response, request)
         def finish_scraping(_):
             slot.finish_response(response, request)
@@ -165,11 +162,9 @@ class Scraper(object):
         from the given spider
         """
         if isinstance(output, Request):
-            self.signals.send_catch_log(signal=signals.request_received, request=output, \
-                spider=spider)
             self.crawler.engine.crawl(request=output, spider=spider)
         elif isinstance(output, BaseItem):
-            self.slots[spider].itemproc_size += 1
+            self.slot.itemproc_size += 1
             dfd = self.itemproc.process_item(output, spider)
             dfd.addBoth(self._itemproc_finished, output, response, spider)
             return dfd
@@ -187,7 +182,10 @@ class Scraper(object):
         """
         if spider_failure is download_failure:
             errmsg = spider_failure.getErrorMessage()
-            if errmsg:
+            if spider_failure.frames:
+                log.err(spider_failure, 'Error downloading %s' % request,
+                        level=log.ERROR, spider=spider)
+            elif errmsg:
                 log.msg(format='Error downloading %(request)s: %(errmsg)s',
                         level=log.ERROR, spider=spider, request=request, errmsg=errmsg)
             return
@@ -196,19 +194,19 @@ class Scraper(object):
     def _itemproc_finished(self, output, item, response, spider):
         """ItemProcessor finished for the given ``item`` and returned ``output``
         """
-        self.slots[spider].itemproc_size -= 1
+        self.slot.itemproc_size -= 1
         if isinstance(output, Failure):
             ex = output.value
             if isinstance(ex, DropItem):
                 logkws = self.logformatter.dropped(item, ex, response, spider)
-                log.msg(level=log.WARNING, spider=spider, **logkws)
+                log.msg(spider=spider, **logkws)
                 return self.signals.send_catch_log_deferred(signal=signals.item_dropped, \
                     item=item, spider=spider, exception=output.value)
             else:
                 log.err(output, 'Error processing %s' % item, spider=spider)
         else:
             logkws = self.logformatter.scraped(output, response, spider)
-            log.msg(level=log.DEBUG, spider=spider, **logkws)
+            log.msg(spider=spider, **logkws)
             return self.signals.send_catch_log_deferred(signal=signals.item_scraped, \
                 item=output, response=response, spider=spider)
 
