@@ -2,12 +2,14 @@
 This module contains essential stuff that should've come with Python itself ;)
 
 It also contains functions (or functionality) which is in Python versions
-higher than 2.5 which is the lowest version supported by Scrapy.
+higher than 2.5 which used to be the lowest version supported by Scrapy.
 
 """
+import os
 import re
 import inspect
 import weakref
+import errno
 from functools import wraps
 from sgmllib import SGMLParser
 
@@ -62,7 +64,7 @@ def unique(list_, key=lambda x: x):
     return result
 
 
-def str_to_unicode(text, encoding=None):
+def str_to_unicode(text, encoding=None, errors='strict'):
     """Return the unicode representation of text in the given encoding. Unlike
     .encode(encoding) this function can be applied directly to a unicode
     object without the risk of double-decoding problems (which can happen if
@@ -72,13 +74,13 @@ def str_to_unicode(text, encoding=None):
     if encoding is None:
         encoding = 'utf-8'
     if isinstance(text, str):
-        return text.decode(encoding)
+        return text.decode(encoding, errors)
     elif isinstance(text, unicode):
         return text
     else:
         raise TypeError('str_to_unicode must receive a str or unicode object, got %s' % type(text).__name__)
 
-def unicode_to_str(text, encoding=None):
+def unicode_to_str(text, encoding=None, errors='strict'):
     """Return the str representation of text in the given encoding. Unlike
     .encode(encoding) this function can be applied directly to a str
     object without the risk of double-decoding problems (which can happen if
@@ -88,7 +90,7 @@ def unicode_to_str(text, encoding=None):
     if encoding is None:
         encoding = 'utf-8'
     if isinstance(text, unicode):
-        return text.encode(encoding)
+        return text.encode(encoding, errors)
     elif isinstance(text, str):
         return text
     else:
@@ -144,19 +146,63 @@ def isbinarytext(text):
     assert isinstance(text, str), "text must be str, got '%s'" % type(text).__name__
     return any(c in _BINARYCHARS for c in text)
 
-def get_func_args(func):
+def get_func_args(func, stripself=False):
     """Return the argument name list of a callable"""
     if inspect.isfunction(func):
         func_args, _, _, _ = inspect.getargspec(func)
+    elif inspect.isclass(func):
+        return get_func_args(func.__init__, True)
+    elif inspect.ismethod(func):
+        return get_func_args(func.__func__, True)
+    elif inspect.ismethoddescriptor(func):
+        return []
     elif hasattr(func, '__call__'):
-        try:
-            func_args, _, _, _ = inspect.getargspec(func.__call__)
-        except Exception:
-            func_args = []
+        if inspect.isroutine(func):
+            return []
+        else:
+            return get_func_args(func.__call__, True)
     else:
         raise TypeError('%s is not callable' % type(func))
+    if stripself:
+        func_args.pop(0)
     return func_args
 
+def get_spec(func):
+    """Returns (args, kwargs) tuple for a function
+    >>> import re
+    >>> get_spec(re.match)
+    (['pattern', 'string'], {'flags': 0})
+
+    >>> class Test(object):
+    ...     def __call__(self, val):
+    ...         pass
+    ...     def method(self, val, flags=0):
+    ...         pass
+
+    >>> get_spec(Test)
+    (['self', 'val'], {})
+
+    >>> get_spec(Test.method)
+    (['self', 'val'], {'flags': 0})
+
+    >>> get_spec(Test().method)
+    (['self', 'val'], {'flags': 0})
+    """
+
+    if inspect.isfunction(func) or inspect.ismethod(func):
+        spec = inspect.getargspec(func)
+    elif hasattr(func, '__call__'):
+        spec = inspect.getargspec(func.__call__)
+    else:
+        raise TypeError('%s is not callable' % type(func))
+
+    defaults = spec.defaults or []
+
+    firstdefault = len(spec.args) - len(defaults)
+    args = spec.args[:firstdefault]
+    kwargs = dict(zip(spec.args[firstdefault:], defaults))
+    return args, kwargs
+
 def equal_attributes(obj1, obj2, attributes):
     """Compare two objects attributes"""
     # not attributes given return False by default
@@ -205,3 +251,29 @@ def stringify_dict(dct_or_tuples, encoding='utf-8', keys_only=True):
             v = v.encode(encoding) if isinstance(v, unicode) else v
         d[k] = v
     return d
+
+def is_writable(path):
+    """Return True if the given path can be written (if it exists) or created
+    (if it doesn't exist)
+    """
+    if os.path.exists(path):
+        return os.access(path, os.W_OK)
+    else:
+        return os.access(os.path.dirname(path), os.W_OK)
+
+def setattr_default(obj, name, value):
+    """Set attribute value, but only if it's not already set. Similar to
+    setdefault() for dicts.
+    """
+    if not hasattr(obj, name):
+        setattr(obj, name, value)
+
+
+def retry_on_eintr(function, *args, **kw):
+    """Run a function and retry it while getting EINTR errors"""
+    while True:
+        try:
+            return function(*args, **kw)
+        except IOError, e:
+            if e.errno != errno.EINTR:
+                raise
